<?php

namespace App\Http\Controllers\API;

use App\Http\Controllers\Controller;
use App\Http\Requests\API\Task\CreateOrUpdateRequest;
use App\Http\Resources\API\TaskDetailResource;
use App\Http\Resources\API\TaskResource;
use App\Models\CustomField;
use App\Models\NamingRule;
use App\Models\Task;
use App\Repositories\CustomFieldRepository;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;

class TaskController extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function index(Request $request)
    {
        $tasks = Task::query()
            ->where("parent_id", 0)
            ->with(['children', 'assignTo', 'createdBy'])
            ->filter($request->all())->paginate();

        return TaskResource::collection($tasks);
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(CreateOrUpdateRequest $request, CustomFieldRepository $customFieldRepo)
    {
        $formData = [
            ...$request->all(),
            'company_id' => Auth::user()->company_id,
            'created_by' => Auth::id(),
            'whitelist' => $request->whitelist ? sprintf(",%s", implode(',', $request->whitelist)) : null,
        ];

        if ($request->has("naming_rule_id") && $request->get("naming_rule_id") > 0) {
            $keys = $customFieldRepo->keysByGroup($request->get("naming_rule_id"));
            $formData['naming_rules'] = $request->only($keys);
        }

        Task::create($formData);

        return $this->created();
    }

    /**
     * Display the specified resource.
     */
    public function show(string $id)
    {
        $task = Task::query()->findOrFail($id);

        return new TaskDetailResource($task);
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(CreateOrUpdateRequest $request, string $id, CustomFieldRepository $customFieldRepo)
    {
        $task = Task::query()->findOrFail($id);

        $formData = [...$request->all(),
            'whitelist' => $request->whitelist ? sprintf(",%s", implode(',', $request->whitelist)) : null,
        ];

        if ($request->has("naming_rule_id") && $request->get("naming_rule_id") > 0) {
            $keys = $customFieldRepo->keysByGroup($request->get("naming_rule_id"));
            $formData['naming_rules'] = $request->only($keys);
        }

        $task->fill($formData);
        $task->save();

        return $this->noContent();
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(string $id)
    {
        $task = Task::query()->findOrFail($id);

        $task->delete();

        return $this->noContent();
    }
}
